from typing import Type, Union, Mapping, Iterable, TypedDict, TYPE_CHECKING
from typing_extensions import override, Self
from dataclasses import dataclass
from nonebot.adapters import Message as BaseMessage, MessageSegment as BaseMessageSegment


class MessageSegment(BaseMessageSegment["Message"]):

    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @override
    def __str__(self) -> str:
        if self.type == "text":
            return self.data["content"]
        return str(self.type)

    @override
    def is_text(self) -> bool:
        return self.type == "text"
    
    @classmethod
    def text(cls, content: str, ats: str = "") -> Self:
        """文本消息
        :param content: 文本内容
        :param ats: 被@的用户列表，格式为`wxid1,wxid2`
        """
        return Text("text", {"content": content, "ats": ats})
    
    @classmethod
    def image(cls, imgUrl: str) -> Self:
        """图片消息
        :param imgUrl: 图片链接
        """
        return Image("image", {"imgUrl": imgUrl})

    @classmethod
    def voice(cls, voiceUrl: str ,voiceDuration: int) -> Self:
        """语音消息
        :param voiceUrl: 语音链接, 仅支持silk格式
        :param voiceDuration: 语音时长, 单位毫秒
        """
        return Voice("voice", {"voiceUrl": voiceUrl, "voiceDuration": voiceDuration})
    
    @classmethod
    def video(cls, videoUrl: str, thumbUrl: str, videoDuration: int) -> Self:
        """视频消息
        :param videoUrl: 视频链接
        :param thumbUrl: 视频缩略图链接
        :param videoDuration: 视频时长, 单位秒
        """
        return Video("video", {"videoUrl": videoUrl, "thumbUrl": thumbUrl, "videoDuration": videoDuration})

    @classmethod
    def file(cls, fileUrl: str, fileName: str) -> Self:
        """文件消息
        :param fileUrl: 文件链接
        :param fileName: 文件名
        """
        return File("file", {"fileUrl": fileUrl, "fileName": fileName})

    @classmethod
    def namecard(cls, nameCardWxid: str, nickName: str) -> Self:
        """名片消息
        :param nameCardWxid: 名片用户wxid
        :param nickName: 名片用户昵称
        """
        return NameCard("namecard", {"nameCardWxid": nameCardWxid, "nickName": nickName})
    
    @classmethod
    def link(cls, title: str, desc: str, linkUrl: str, thumbUrl: str) -> Self:
        """链接消息
        :param title: 链接标题
        :param desc: 链接描述
        :param linkUrl: 链接地址
        :param thumbUrl: 链接缩略图链接
        """
        return Link("link", {"title": title, "desc": desc, "linkUrl": linkUrl, "thumbUrl": thumbUrl})

    @classmethod
    def emoji(cls, emojiMd5: str, emojiSize: int) -> Self:
        """表情消息
        :param emojiMd5: 表情md5
        :param emojiSize: 表情大小
        """
        return Emoji("emoji", {"emojiMd5": emojiMd5, "emojiSize": emojiSize})
    
    @classmethod
    def appmsg(cls, appmsg: str) -> Self:
        """公众号消息
        :param appmsg: 回调消息中的appmsg节点内容
        """
        return AppMsg("appmsg", {"appmsg": appmsg})
    
    @classmethod
    def miniapp(cls, miniAppId: str, displayName: str, pagePath: str, coverImgUrl: str, title: str, userName: str) -> Self:
        """小程序消息
        :param miniAppId: 小程序id
        :param displayName: 小程序名称
        :param pagePath: 小程序打开的地址
        :param coverImgUrl: 小程序封面图片链接
        :param title: 小程序标题
        :param userName: 归属的用户ID
        """
        return MiniApp("mp", {"miniAppId": miniAppId, "displayName": displayName, "pagePath": pagePath, "coverImgUrl": coverImgUrl, "title": title, "userName": userName})
    
    @classmethod
    def revoke(cls, msgId: str, newMsgId: str, createTime: str) -> Self:
        """撤回消息
        :param msgId: 回调消息中的msgId
        :param newMsgId: 回调消息中的newMsgId
        :param createTime: 回调消息中的createTime
        """
        return Revoke("revoke", {"msgId": msgId, "newMsgId": newMsgId, "createTime": createTime})
    
    @classmethod
    def forwardFile(cls, xml: str) -> Self:
        """转发文件
        :param xml: 文件消息的xml
        """
        return forwardFile("forwardFile", {"xml": xml})
    
    @classmethod
    def forwardImage(cls, xml: str) -> Self:
        """转发图片
        :param xml: 图片消息的xml
        """
        return forwardImage("forwardImage", {"xml": xml})
    
    @classmethod
    def forwardVideo(cls, xml: str) -> Self:
        """转发视频
        :param xml: 视频消息的xml
        """
        return forwardVideo("forwardVideo", {"xml": xml})
    
    @classmethod
    def forwardLink(cls, xml: str) -> Self:
        """转发链接
        :param xml: 链接消息的xml
        """
        return forwardLink("forwardLink", {"xml": xml})
    
    @classmethod
    def forwardMP(cls, xml: str, coverImgUrl: str) -> Self:
        """转发小程序
        :param xml: 小程序消息的xml
        :param coverImgUrl: 小程序封面图片链接
        """
        return forwardMP("forwardMP", {"xml": xml, "coverImgUrl": coverImgUrl})

    @classmethod
    def xml(cls, xml: str) -> Self:
        """xml消息
        :param xml: xml
        """
        return Xml("xml", {"xml": xml})

class Message(BaseMessage[MessageSegment]):

    @classmethod
    @override
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @staticmethod
    @override
    def _construct(msg: str) -> Iterable[MessageSegment]:
        return [MessageSegment.text(msg)]

class _TextData(TypedDict):
    content: str
    ats: list

@dataclass
class Text(MessageSegment):
    if TYPE_CHECKING:
        data: _TextData

    @override
    def __str__(self):
        return self.data["content"]
    
    @override
    def is_text(self) -> bool:
        return True
    
class _ImageData(TypedDict):
    imgUrl: str

@dataclass
class Image(MessageSegment):
    if TYPE_CHECKING:
        data: _ImageData

class _VoiceData(TypedDict):
    voiceUrl: str
    voiceDuration: int

@dataclass
class Voice(MessageSegment):
    if TYPE_CHECKING:
        data: _VoiceData

class _VideoData(TypedDict):
    videoUrl: str
    thumbUrl: str
    videoDuration: int

@dataclass
class Video(MessageSegment):
    if TYPE_CHECKING:
        data: _VideoData

class _FileData(TypedDict):
    fileUrl: str
    fileName: str

@dataclass
class File(MessageSegment):
    if TYPE_CHECKING:
        data: _FileData

class _NameCardData(TypedDict):
    nameCardWxid: str
    nickName: str

@dataclass
class NameCard(MessageSegment):
    if TYPE_CHECKING:
        data: _NameCardData

class _LinkData(TypedDict):
    title: str
    desc: str
    linkUrl: str
    thumbUrl: str

@dataclass
class Link(MessageSegment):
    if TYPE_CHECKING:
        data: _LinkData

class _EmojiData(TypedDict):
    emojiMd5: str
    emojiSize: str

@dataclass
class Emoji(MessageSegment):
    if TYPE_CHECKING:
        data: _EmojiData

class _AppMsg(TypedDict):
    appmsg: str

@dataclass
class AppMsg(MessageSegment):
    if TYPE_CHECKING:
        data: _AppMsg

class _MiniAppData(TypedDict):
    miniAppId: str
    displayName: str
    pagePath: str
    coverImgUrl: str
    title: str
    userName: str

@dataclass
class MiniApp(MessageSegment):
    if TYPE_CHECKING:
        data: _MiniAppData

class _RevokeData(TypedDict):
    msgId: str
    newMsgId: str
    createTime: int

@dataclass
class Revoke(MessageSegment):
    if TYPE_CHECKING:
        data: _RevokeData

class _XmlData(TypedDict):
    xml: str

@dataclass
class Xml(MessageSegment):
    if TYPE_CHECKING:
        data: _XmlData

@dataclass
class forwardFile(MessageSegment):
    if TYPE_CHECKING:
        data: _XmlData

@dataclass
class forwardImage(MessageSegment):
    if TYPE_CHECKING:
        data: _XmlData

@dataclass
class forwardVideo(MessageSegment):
    if TYPE_CHECKING:
        data: _XmlData

@dataclass
class forwardLink(MessageSegment):
    if TYPE_CHECKING:
        data: _XmlData

class _ForwardMiniAppData(TypedDict):
    xml: str
    coverImgUrl: str

@dataclass
class forwardMP(MessageSegment):
    if TYPE_CHECKING:
        data: _ForwardMiniAppData



from pydantic import BaseModel
from typing import List, Dict, Union, Optional

# 联系人模块

class InfoData(BaseModel):
    """好友/群聊数据"""
    # 用户名
    userName: str
    nickName: str
    pyInitial: str
    quanPin: str
    sex: int
    remark: str
    remarkPyInitial: str
    remarkQuanPin: str
    signature: Optional[str] = None
    alias: str
    snsBgImg: Optional[str] = None
    country: str
    bigHeadImgUrl: str
    smallHeadImgUrl: str
    description: Optional[str] = None
    cardImgUrl: Optional[str] = None
    labelList: Optional[str] = None
    province: str
    city: str
    phoneNumList: Optional[List[str]] = None

class Response(BaseModel):
    ret: int
    """状态码"""
    msg: str
    """返回信息"""
    data: Optional[Dict] = None
    """返回数据"""

class ContactListData(BaseModel):
    """联系人列表数据"""
    friends: List[str]
    """好友列表"""
    chatrooms: List[str]
    """群聊列表"""
    ghs: List[str]
    """公众号列表"""

class ContactListResponse(Response):
    data: ContactListData

class SearchRequest(BaseModel):
    """搜索请求"""
    contactsInfo: str
    """搜索的联系人信息, 微信号、手机号..."""

class SearchSuccessData(BaseModel):
    """搜索成功数据"""
    v3: str
    nickName: str
    """昵称"""
    sex: str
    """性别"""
    signature: str
    """个性签名"""
    bigHeadImgUrl: str
    """头像大图"""
    smallHeadImgUrl: str
    """头像小图"""
    v4: str

class SearchFailureData(BaseModel):
    code: str
    msg: str

class SearchResponse(Response):
    data: Union[SearchSuccessData, SearchFailureData]

class AddContactRequest(BaseModel):
    """添加联系人请求"""
    scene: str
    """添加来源"""
    option: str
    """操作类型"""
    v3: str
    v4: str
    content: str
    """添加好友时的招呼语"""

class DeleteFriendRequest(BaseModel):
    """删除好友请求"""
    wxid: str
    """好友的微信号"""

class uploadPhoneAddressRequest(BaseModel):
    """上传手机通讯录请求"""
    phones: List[str]
    """需要上传的手机号"""
    opType: int
    """操作类型, 1:添加, 2:删除"""

class GetBreifInfoRequest(BaseModel):
    """获取简要信息请求"""
    wxids: List[str]
    """好友的微信号,>=1 <=100"""

class GetBreifInfoResponse(Response):
    data: List[InfoData]

class GetDetailInfoRequest(BaseModel):
    """获取详细信息请求"""
    wxids: List[str]
    """好友的微信号,>=1 <=20"""

class GetDetailInfoResponse(Response):
    data: List[InfoData]

class SetFriendPermissionsRequest(BaseModel):
    """设置好友权限请求"""
    wxid: str
    """好友的微信号"""
    onlyChat: bool
    """设置好友是否仅聊天"""

class SetFriendRemarkRequest(BaseModel):
    """设置好友备注请求"""
    wxid: str
    """好友的微信号"""
    remark: str
    """备注"""

class GetPhoneAddressListRequest(BaseModel):
    """获取手机通讯录请求"""
    phones: Optional[List[str]] = None
    """获取哪些手机号的好友详情, 不传获取所有"""

class GetPhoneAddressListResponse(Response):
    data: List[InfoData]

# 群模块

class createChatroomRequest(BaseModel):
    """创建群聊请求"""
    wxids: List[str]
    """群聊成员的微信号,>=2"""

class createChatroomData(BaseModel):
    """创建群聊数据"""
    headImgBase64: str
    """群聊头像的base64编码"""
    chatroomId: str
    """群聊ID"""

class createChatroomResponse(Response):
    data: createChatroomData

class modifyChatroomNameRequest(BaseModel):
    """修改群聊请求"""
    chatroomName: str
    """群聊ID"""
    chatroomId: str
    """群聊名称"""

class modifyChatroomRemarkRequest(BaseModel):
    """修改群聊备注请求"""
    chatroomId: str
    """群聊ID"""
    chatroomRemark: str
    """备注"""

class modifyChatroomNickNameForSelfRequest(BaseModel):
    """修改群聊昵称请求"""
    chatroomId: str
    """群聊ID"""
    nickName: str
    """昵称"""

class inviteMemberRequest(BaseModel):
    """邀请成员请求"""
    chatroomId: str
    """群聊ID"""
    wxids: str
    """成员的微信号,用英文逗号分隔"""
    reason: str
    """邀请理由"""

class removeMemberRequest(BaseModel):
    """移除成员请求"""
    chatroomId: str
    """群聊ID"""
    wxids: str
    """成员的微信号,用英文逗号分隔"""

class quitChatroomRequest(BaseModel):
    """退出群聊请求"""
    chatroomId: str
    """群聊ID"""

class disbandChatroomRequest(BaseModel):
    """解散群聊请求"""
    chatroomId: str
    """群聊ID"""

class getChatroomInfoRequest(BaseModel):
    """获取群聊信息请求"""
    chatroomId: str
    """群聊ID"""

class ChatroomMemberInfo(BaseModel):
    """群成员信息数据"""
    wxid: str
    """成员的wxid"""
    nickName: str
    """成员的昵称"""
    inviterUserName: Optional[str] = None
    """邀请人的wxid"""
    memberFlag: Optional[int] = None
    """标识"""
    displayName: Optional[str] = None
    """在本群内的昵称"""
    bigHeadImgUrl: Optional[str] = None
    """大尺寸头像"""
    smallHeadImgUrl: Optional[str] = None
    """小尺寸头像"""

class getChatroomInfoData(BaseModel):
    """获取群聊信息数据"""
    chatroomId: str
    """群聊ID"""
    nickName: str
    """群聊名称"""
    pyInitial: str
    """群聊名称的拼音首字母"""
    quanPin: str
    """群聊名称的拼音全拼"""
    sex: int
    remark: str
    """群备注, 仅自己可见"""
    remarkPyInitial: str
    """备注的拼音首字母"""
    remarkQuanPin: str
    """备注的拼音全拼"""
    chatRoomNotify: int
    """群消息是否提醒"""
    chatRoomOwner: str
    """群主的wxid"""
    smallHeadImgUrl: int
    """群头像链接"""
    memberList: List[ChatroomMemberInfo]
    """群成员信息"""

class getChatroomInfoResponse(Response):
    data: getChatroomInfoData

class getChatroomMemberListRequest(BaseModel):
    """获取群成员列表请求"""
    chatroomId: str
    """群聊ID"""
    
class getChatroomMemberListData(BaseModel):
    """获取群成员列表数据"""
    memberList: List[ChatroomMemberInfo]
    """群成员信息"""
    chatroomOwner: Optional[str] = None
    """群主wxid"""
    adminWxid: Optional[List[Dict[str, str]]] = None
    """管理员wxid"""
    
class getChatroomMemberListResponse(Response):
    data: getChatroomMemberListData

class getChatroomMemberDetailRequest(BaseModel):
    """获取群成员详情请求"""
    chatroomId: str
    """群聊ID"""
    memberWxids: List[str]
    """成员的wxid"""

class MemberDetail(BaseModel):
    """成员详情数据"""
    userName: str
    """成员的wxid"""
    nickName: str
    """成员的昵称"""
    pyInitial: str
    """成员昵称的拼音首字母"""
    quanPin: str
    """成员昵称的拼音全拼"""
    sex: int
    """成员的性别"""
    remark: Optional[str] = None
    """成员的备注"""
    remarkPyInitial: Optional[str] = None
    """成员备注的拼音首字母"""
    remarkQuanPin: Optional[str] = None
    """成员备注的拼音全拼"""
    chatRoomNotify: int
    """群消息是否提醒"""
    signature: Optional[str] = None
    """个性签名"""
    alias: Optional[str] = None
    """微信号"""
    snsBgImg: str
    """朋友圈背景图片"""
    bigHeadImgUrl: str
    """大尺寸头像"""
    smallHeadImgUrl: str
    """小尺寸头像"""
    description: Optional[str] = None
    """描述"""
    cardImgUrl: Optional[str] = None
    """描述的图片链接"""
    labelList: Optional[str] = None
    """标签列表, 多个英文逗号分隔"""
    country: str
    """国家"""
    province: Optional[str] = None
    """省份"""
    city: Optional[str] = None
    """城市"""
    phoneNumList: Optional[List[str]] = None
    """手机号码"""
    friendUserName: str
    """好友wxid"""
    inviterUserName: Optional[str] = None
    """邀请人的wxid"""
    memberFlag: Optional[str] = None
    """标识"""

class getChatroomMemberDetailResponse(Response):
    data: List[MemberDetail]

class getChatroomAnnouncementRequest(BaseModel):
    """获取群公告请求"""
    chatroomId: str
    """群聊ID"""

class ChatroomAnnouncement(BaseModel):
    """群公告数据"""
    announcement: str
    """公告内容"""
    announcementEditor: int
    """群公告作者的wxid"""
    publishTime: int
    """群公告发布时间"""

class getChatroomAnnouncementResponse(Response):
    data: ChatroomAnnouncement

class setChatroomAnnouncementRequest(BaseModel):
    """设置群公告请求"""
    chatroomId: str
    """群聊ID"""
    content: str
    """公告内容"""

class agreeJoinRoomRequest(BaseModel):
    """同意加群请求"""
    url: str
    """邀请进群回调消息中的url"""

class agreeJoinRoomData(BaseModel):
    chatroomId: str

class agreeJoinRoomResponse(Response):
    data: agreeJoinRoomData

class addGroupMemberAsFriendRequest(BaseModel):
    """添加群成员为好友请求"""
    chatroomId: str
    """群聊ID"""
    memberWxid: str
    """成员的wxid"""
    content: str
    """加好友的招呼语"""

class addGroupMemberAsFriendData(BaseModel):
    """添加群成员为好友数据"""
    v3: str
    """添加群成员的v3, 通过好友后会通过回调消息返回此值"""

class addGroupMemberAsFriendResponse(Response):
    data: addGroupMemberAsFriendData

class getChatroomQrCodeRequest(BaseModel):
    """获取群二维码请求"""
    chatroomId: str
    """群聊ID"""

class getChatroomQrCodeData(BaseModel):
    """获取群二维码数据"""
    qrBase64: str
    """群二维码图片的base64"""
    qrTips: str
    """群二维码的提示信息"""

class getChatroomQrCodeResponse(Response):
    data: getChatroomQrCodeData

class saveContractListRequest(BaseModel):
    """群保存到通讯录请求"""
    chatroomId: str
    """群聊ID"""
    operType: int
    """操作类型 3保存到通讯录 2从通讯录移除"""

class adminOperateRequest(BaseModel):
    """管理员操作请求"""
    chatroomId: str
    """群聊ID"""
    operType: str
    """操作类型 1: 添加群管理(可添加多个微信号) 2: 删除群管理(可删除多个) 3: 转让(只能转让一个微信号)"""
    wxids: List[str]
    """操作对象的wxid"""

class pinChatRequest(BaseModel):
    """置顶聊天请求"""
    chatroomId: str
    """群聊ID"""
    top: bool
    """是否置顶"""

class setMsgSilenceRequest(BaseModel):
    """消息免打扰请求"""
    chatroomId: str
    """群聊ID"""
    silence: bool
    """是否免打扰"""

class joinRoomUsingQRCodeRequest(BaseModel):
    """扫码进群"""
    qrUrl: str
    """通过解析群二维码图片获得的链接"""

class joinRoomUsingQRCodeData(BaseModel):
    """扫码进群数据"""
    chatroomName: str
    """群聊名称"""
    html: Optional[str] = None
    chatroomId: str
    """群聊ID"""

class joinRoomUsingQRCodeResponse(Response):
    data: joinRoomUsingQRCodeData

class roomAccessApplyCheckApproveRequest(BaseModel):
    """群申请审核"""
    chatroomId: str
    """群聊ID"""
    wnewMsgIdxid: str
    """消息ID"""
    msgContent: str
    """消息内容"""

# 消息模块(此处只实现撤回方便使用、其余部分在适配器已实现)

class revokeMsgRequest(BaseModel):
    """撤回消息请求"""
    toWxid: str
    """好友/群的ID"""
    msgId: str
    """发送类接口返回的msgId"""
    newMsgId: str
    """发送类接口返回的newMsgId"""
    createTime: str
    """发送类接口返回的createTime"""

# 标签模块

class Label(BaseModel):
    """标签数据"""
    labelId: int
    """标签ID"""
    labelName: str
    """标签名称"""

class addLabelRequest(BaseModel):
    """添加标签请求"""
    labelName: str
    """标签名称"""

class addLabelResponse(Response):
    data: Label

class delLabelRequest(BaseModel):
    """删除标签请求"""
    labels: str
    """标签ID, 多个逗号分隔"""

class getLabelListData(BaseModel):
    """获取标签列表数据"""
    labelList: List[Label]
    """标签列表"""

class getLabelListResponse(Response):
    data: getLabelListData

class modifyMemberListRequest(BaseModel):
    """修改好友标签,每次在修改时都需要进行全量修改"""
    labelIds: str
    """标签ID, 多个逗号分隔"""
    wxIds: List[str]
    """好友wxid"""

# 个人模块

class profileRequest(BaseModel):
    """获取个人资料请求"""
    proxyIp: str = ""
    """proxyIp"""

class Profile(BaseModel):
    """个人资料"""
    alias: Optional[str] = None
    """微信号"""
    wxid: str
    """wxid"""
    nickName: str
    """昵称"""
    mobile: str
    """手机号"""
    uin: int
    """uin"""
    sex: int
    """性别"""
    province: Optional[str] = None
    """省份"""
    city: Optional[str] = None
    """城市"""
    signature: Optional[str] = None
    """个性签名"""
    country: Optional[str] = None
    """国家"""
    bigHeadImgUrl: Optional[str] = None
    """大尺寸头像"""
    smallHeadImgUrl: Optional[str] = None
    """小尺寸头像"""
    regCountry: str
    """注册国家"""
    snsBgImg: Optional[str] = None
    """朋友圈背景图"""

class getProfileResponse(Response):
    data: Profile

class getQrCodeData(BaseModel):
    """获取二维码数据"""
    qrCode: str
    """二维码图片的base64"""

class getQrCodeResponse(Response):
    data: getQrCodeData

class SafetyInfoData(BaseModel):
    """获取设备记录"""
    uuid: str
    """设备uuid"""
    deviceName: str
    """设备名称"""
    deviceType: str
    """设备类型"""
    lastTime: int
    """最后使用时间"""

class getSafetyInfoData(BaseModel):
    """获取设备记录请求"""
    list: List[SafetyInfoData]

class getSafetyInfoResponse(Response):
    data: getSafetyInfoData

class privacySettingsRequest(BaseModel):
    """隐私设置请求"""
    option: int
    """
    隐私设置的选项
    4: 加我为朋友时需要验证
    7: 向我推荐通讯录朋友
    8: 添加我的方式 手机号
    25: 添加我的方式 微信号
    38: 添加我的方式 群聊
    39: 添加我的方式 我的二维码
    40: 添加我的方式 名片
    """
    open: bool
    """开关"""

class updateProfileRequest(BaseModel):
    """修改个人资料请求"""
    city: str
    """城市"""
    country: str
    """国家"""
    nickName: str
    """昵称"""
    province: str
    """省份"""
    signature: str
    """个性签名"""
    sex: int
    """性别"""

class updateHeadImgRequest(BaseModel):
    """修改头像请求"""
    headImgUrl: str
    """头像的图片地址"""

# 收藏夹模块

class FavorFold(BaseModel):
    """收藏夹"""
    favId: str
    """收藏夹ID"""
    type: str
    """收藏夹类型"""
    flag: str
    """收藏夹标识"""
    updateTime: str
    """更新时间"""

class FavorItem(BaseModel):
    """收藏夹内容"""
    favId: str
    """收藏夹ID"""
    status: str
    """状态"""
    flag: str
    """收藏夹标识"""
    updateTime: str
    """更新时间"""
    content: str
    """收藏夹内容"""

class syncFavorRequest(BaseModel):
    """同步收藏夹请求,响应结果中会包含已删除的的收藏夹记录, 通过flag=1来判断已删除"""
    syncKey: str
    """翻页key, 首次传空, 获取下一页传接口返回的syncKey"""

class syncFavorData(BaseModel):
    syncKey: str
    list: Optional[List[FavorFold]] = None

class syncFavorResponse(Response):
    data: syncFavorData

class getFavorContentRequest(BaseModel):
    """获取收藏夹内容请求"""
    favId: str
    """收藏夹ID"""

class getFavorContentResponse(Response):
    data: FavorItem

class deleteFavorFolderRequest(BaseModel):
    """删除收藏夹请求"""
    favId: str
    """收藏夹ID"""



import re
import asyncio

from typing import TYPE_CHECKING, Any, Union
from typing_extensions import override

from nonebot.adapters import Bot as BaseBot
from nonebot.message import handle_event
from nonebot.compat import type_validate_python, model_dump
from nonebot.adapters.gewechat.event import Event
from .message import Message, MessageSegment
from .event import Event, TextMessageEvent
from .utils import log, resp_json
from .api_model import *

if TYPE_CHECKING:
    from .adapter import Adapter
    

def check_at_me(bot: "Bot", event: TextMessageEvent):
    if "notify@all" in event.at_list:
        event.to_me = True
    if bot.config.nickname:
        nickname_regex = "|".join(bot.config.nickname)
        m = re.search(rf"^@({nickname_regex})([\s,, ]*|$)", event.msg, re.IGNORECASE)
        if m:
            nickname = m.group(1).strip()
            log("DEBUG", f"User is at me: {nickname}")
            event.to_me = True
            loc = m.end()
            event.msg = event.msg[loc:]
            event.message[0].data["content"] = event.msg

def check_nickname(bot: "Bot", event: TextMessageEvent):
    nicknames = bot.config.nickname
    nickname_regex = "|".join(nicknames)
    m = re.search(rf"^({nickname_regex})([\s,, ]*|$)", event.msg, re.IGNORECASE)
    if m:
        nickname = m.group(1).strip()
        log("DEBUG", f"User is calling me: {nickname}")
        event.to_me = True
        loc = m.end()
        event.msg = event.msg[loc:]
        event.message[0].data["content"] = event.msg

def to_me(bot: "Bot", event: TextMessageEvent):
    check_at_me(bot, event)
    check_nickname(bot, event)

class Bot(BaseBot):
    """Gewechat Bot 适配"""

    @override
    def __init__(self, adapter, self_id: str, **kwargs: Any):
        super().__init__(adapter, self_id)
        self.adapter: Adapter = adapter

    async def handle_event(self, event: Event):
        # 根据需要, 对事件进行某些预处理, 例如：
        # 检查事件是否和机器人有关操作, 去除事件消息首尾的 @bot
        # 检查事件是否有回复消息, 调用平台 API 获取原始消息的消息内容
        if isinstance(event, TextMessageEvent):
            to_me(self, event)
        # 调用 handle_event 让 NoneBot 对事件进行处理
        await handle_event(self, event)

    async def call_api(self, api: str, **data: Any) -> Response:
        if not data.get("appId"):
            data["appId"] = self.config.appid
        return await self.adapter._do_call_api(api, **data)

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs,
    ) -> Any:
        try:
            toWxid = getattr(event, "FromUserName")
        except AttributeError:
            raise ValueError("该事件不支持发送消息")
        if isinstance(message, str) or isinstance(message, MessageSegment):
            message = Message(message)


        api_map = {
            "text": "Text",
            "image": "Image",
            "file": "File",
            "video": "Video",
            "voice": "Voice",
            "emoji": "Emoji",
            "namecard": "NameCard",
            "appmsg": "AppMsg",
            "mp": "MiniApp"
        }

        msg_list = []
        for segment in message:
            if segment.type == "revoke":
                api = "/message/revokeMsg"
            elif "forward" in segment.type:
                api = f"/message/{segment.type}"
            else:
                api = f"/message/post{api_map[segment.type]}"

            segment.data["toWxid"] = toWxid
            segment.data["appId"] = self.adapter.adapter_config.appid

            msg_list.append(self.call_api(api, **segment.data))

        return await asyncio.gather(*msg_list)
    
    async def check_online(self) -> bool:
        """检查是否在线"""
        return resp_json(await self.call_api("/login/checkOnline"))['data']
    
    async def reconnect(self) -> dict:
        """重连"""
        return resp_json(await self.call_api("/login/reconnection"))
    
    async def logout(self) -> dict:
        """登出"""
        self.adapter.bot_disconnect(self)
        return resp_json(await self.call_api("/login/logout"))
    
    async def fetchContactsList(self, cache: bool = False) -> ContactListResponse:
        """
        获取联系人列表
        cache: 是否使用缓存, 默认为 False
        """
        if cache:
            self.call_api("/contacts/fetchContactsListCache")
        return type_validate_python(ContactListResponse, resp_json(await self.call_api("/contacts/fetchContactsList")))

    async def search(self, keyword: str) -> SearchResponse:
        """
        搜索联系人
        搜索的联系人信息若已经是好友, 响应结果的v3则为好友的wxid
        本接口返回的数据可通过添加联系人接口发送添加好友请求
        keyword: 搜索的联系人信息, 微信号、手机号...
        """
        request = SearchRequest(contactsInfo=keyword)
        return type_validate_python(SearchResponse, resp_json(await self.call_api("/contacts/search", **model_dump(request))))

    async def addContact(self, scene: int, option: int, v3: str, v4: str, content: str) -> Response:
        """
        添加/同意添加联系人
        本接口建议在线3天后再进行调用。
        好友添加成功后, 会通过回调消息推送一条包含v3的消息, 可用于判断好友是否添加成功。
        scene: 添加来源
        option: 操作类型
        v3: 通过搜索或回调消息获取到的v3
        v4: 通过搜索或回调消息获取到的v4
        content: 添加好友时的招呼语
        """
        request = AddContactRequest(scene=scene, option=option, v3=v3, v4=v4, content=content)
        return type_validate_python(Response, resp_json(await self.call_api("/contacts/addContacts", **model_dump(request))))
    
    async def deleteFriend(self, wxid: str) -> Response:
        """
        删除联系人
        wxid: 联系人v3
        """
        request = DeleteFriendRequest(wxid=wxid)
        return type_validate_python(Response, resp_json(await self.call_api("/contacts/deleteFriend", **model_dump(request))))
    
    async def uploadPhoneAddressList(self, phones: List[str], opType: int) -> Response:
        """
        上传手机通讯录
        phones: 需要上传的手机号
        opType: 操作类型, 1:上传 2:删除
        """
        request = uploadPhoneAddressRequest(phones=phones, opType=opType)
        return type_validate_python(Response, resp_json(await self.call_api("/contacts/uploadPhoneAddressList", **model_dump(request))))
    
    async def getBreifInfo(self, wxids: List[str]) -> GetBreifInfoResponse:
        """
        获取联系人简要信息
        wxids: 好友的wxid(>=1, <=100)
        """
        if len(wxids) > 100 or len(wxids) == 0:
            log("error", "wxid数量错误", ValueError("wxid数量错误"))
        request = GetBreifInfoRequest(wxids=wxids)
        return type_validate_python(GetBreifInfoResponse, resp_json(await self.call_api("/contacts/getBriefInfo", **model_dump(request))))


    async def getDetailInfo(self, wxids: List[str]) -> GetDetailInfoResponse:
        """
        获取联系人详细信息
        wxid: 好友的wxid(>=1, <=20)
        """
        if len(wxids) > 20 or len(wxids) == 0:
            log("error", "wxid数量错误", ValueError("wxid数量错误"))
        request = GetDetailInfoRequest(wxids=wxids)
        return type_validate_python(GetDetailInfoResponse, resp_json(await self.call_api("/contacts/getDetailInfo", **model_dump(request))))

    async def setFriendPermissions(self, wxid: str, onlyChat: bool) -> Response:
        """
        设置好友仅聊天
        wxid: 好友的wxid
        onlyChat: 是否仅聊天
        """
        request = SetFriendPermissionsRequest(wxid=wxid, onlyChat=onlyChat)
        return type_validate_python(Response, resp_json(await self.call_api("/contacts/setFriendPermissions", **model_dump(request))))
    
    async def setFriendRemark(self, wxid: str, remark: str) -> Response:
        """
        设置好友备注
        wxid: 好友的wxid
        remark: 备注
        """
        request = SetFriendRemarkRequest(wxid=wxid, remark=remark)
        return type_validate_python(Response, resp_json(await self.call_api("/contacts/setFriendRemark", **model_dump(request))))
    
    async def getPhoneAddressList(self, phones: Optional[List[str]] = None) -> GetPhoneAddressListResponse:
        """
        获取手机通讯录
        phones: 获取哪些手机号的好友详情, 不传获取所有
        """
        request = GetPhoneAddressListRequest(phones=phones)
        return type_validate_python(GetPhoneAddressListResponse, resp_json(await self.call_api("/contacts/getPhoneAddressList"), **model_dump(request)))
    
    async def createChatroom(self, wxids: List[str]) -> createChatroomResponse:
        """
        创建群聊
        wxids: 群聊成员的wxid(>=2)
        """
        if len(wxids) < 2:
            log("error", "wxid数量错误", ValueError("wxid数量错误"))
        request = createChatroomRequest(wxids=wxids)
        return type_validate_python(createChatroomResponse, resp_json(await self.call_api("/group/createChatroom", **model_dump(request))))
    
    async def modifyChatroomName(self, chatroomId: str, chatroomName: str) -> Response:
        """
        修改群聊名称
        chatroomId: 群聊id
        chatroomName: 群聊名称
        """
        request = modifyChatroomNameRequest(chatroomId=chatroomId, chatroomName=chatroomName)
        return type_validate_python(Response, resp_json(await self.call_api("/group/modifyChatroomName", **model_dump(request))))
    
    async def modifyChatroomRemark(self, chatroomId: str, chatroomRemark: str) -> Response:
        """
        修改群聊备注,群备注仅自己可见
        chatroomId: 群聊id
        chatroomRemark: 群聊备注
        """
        request = modifyChatroomRemarkRequest(chatroomId=chatroomId, chatroomRemark=chatroomRemark)
        return type_validate_python(Response, resp_json(await self.call_api("/group/modifyChatroomRemark", **model_dump(request))))

    async def modifyChatroomNickNameForSelf(self, chatroomId: str, nickName: str) -> Response:
        """
        修改我在群内的昵称
        chatroomId: 群聊id
        nickName: 昵称
        """
        request = modifyChatroomNickNameForSelfRequest(chatroomId=chatroomId, nickName=nickName)
        return type_validate_python(Response, resp_json(await self.call_api("/group/modifyChatroomNickNameForSelf", **model_dump(request))))

    async def inviteMember(self, chatroomId: str, wxids: List[str], reason: str) -> Response:
        """
        邀请好友入群
        chatroomId: 群聊id
        wxids: 好友wxid
        reason: 邀请理由
        """
        wxids = ",".join(wxids)
        request = inviteMemberRequest(chatroomId=chatroomId, wxids=wxids, reason=reason)
        return type_validate_python(Response, resp_json(await self.call_api("/group/inviteMember", **model_dump(request))))
    
    async def removeMember(self, chatroomId: str, wxids: List[str]) -> Response:
        """
        移除群成员
        chatroomId: 群聊id
        wxids: 好友wxid
        """
        wxids = ",".join(wxids)
        request = removeMemberRequest(chatroomId=chatroomId, wxids=wxids)
        return type_validate_python(Response, resp_json(await self.call_api("/group/removeMember", **model_dump(request))))
    
    async def quitChatroom(self, chatroomId: str) -> Response:
        """
        退出群聊
        chatroomId: 群聊id
        """
        request = quitChatroomRequest(chatroomId=chatroomId)
        return type_validate_python(Response, resp_json(await self.call_api("/group/quitChatroom", **model_dump(request))))
    
    async def disbandChatroom(self, chatroomId: str) -> Response:
        """
        解散群聊
        chatroomId: 群聊id
        """
        request = disbandChatroomRequest(chatroomId=chatroomId)
        return type_validate_python(Response, resp_json(await self.call_api("/group/disbandChatroom", **model_dump(request))))
    
    async def getChatroomInfo(self, chatroomId: str) -> getChatroomInfoResponse:
        """
        获取群聊信息
        chatroomId: 群聊id
        """
        request = getChatroomInfoRequest(chatroomId=chatroomId)
        return type_validate_python(getChatroomInfoResponse, resp_json(await self.call_api("/group/getChatroomInfo", **model_dump(request))))
    
    async def getChatroomMemberList(self, chatroomId: str) -> getChatroomMemberListResponse:
        """
        获取群聊成员列表
        chatroomId: 群聊id
        """
        request = getChatroomMemberListRequest(chatroomId=chatroomId)
        return type_validate_python(getChatroomMemberListResponse, resp_json(await self.call_api("/group/getChatroomMemberList", **model_dump(request))))
    
    async def getChatroomMemberDetail(self, chatroomId: str, memberWxids: List[str]) -> getChatroomMemberDetailResponse:
        """
        获取群聊成员详细信息
        chatroomId: 群聊id
        memberWxids: 成员wxid
        """
        request = getChatroomMemberDetailRequest(chatroomId=chatroomId, memberWxids=memberWxids)
        return type_validate_python(getChatroomMemberDetailResponse, resp_json(await self.call_api("/group/getChatroomMemberDetail", **model_dump(request))))
    
    async def getChatroomAnnouncement(self, chatroomId: str) -> getChatroomAnnouncementResponse:
        """
        获取群公告
        chatroomId: 群聊id
        """
        request = getChatroomAnnouncementRequest(chatroomId=chatroomId)
        return type_validate_python(getChatroomAnnouncementResponse, resp_json(await self.call_api("/group/getChatroomAnnouncement", **model_dump(request))))
    
    async def setChatroomAnnouncement(self, chatroomId: str, content: str) -> Response:
        """
        设置群公告
        chatroomId: 群聊id
        content: 公告内容
        """
        request = setChatroomAnnouncementRequest(chatroomId=chatroomId, content=content)
        return type_validate_python(Response, resp_json(await self.call_api("/group/setChatroomAnnouncement", **model_dump(request))))
    
    async def agreeJoinRoom(self, url: str) -> agreeJoinRoomResponse:
        """
        同意入群申请
        url: 邀请进群回调消息中的url
        """
        request = agreeJoinRoomRequest(url=url)
        return type_validate_python(agreeJoinRoomResponse, resp_json(await self.call_api("/group/agreeJoinRoom", **model_dump(request))))

    async def addGroupMemberAsFriend(self, chatroomId: str, memberWxid: str, content: str) -> addGroupMemberAsFriendResponse:
        """
        添加群成员为好友
        chatroomId: 群聊id
        memberWxid: 成员wxid
        content: 添加好友申请内容
        """
        request = addGroupMemberAsFriendRequest(chatroomId=chatroomId, memberWxid=memberWxid, content=content)
        return type_validate_python(addGroupMemberAsFriendResponse, resp_json(await self.call_api("/group/addGroupMemberAsFriend", **model_dump(request))))
    
    async def getChatroomQrCode(self, chatroomId: str) -> getChatroomQrCodeResponse:
        """
        获取群聊二维码
        chatroomId: 群聊id
        """
        request = getChatroomQrCodeRequest(chatroomId=chatroomId)
        return type_validate_python(getChatroomQrCodeResponse, resp_json(await self.call_api("/group/getChatroomQrCode", **model_dump(request))))
    
    async def saveContractList(self, chatroomId: str, operType: str) -> Response:
        """
        保存群聊到通讯录
        chatroomId: 群聊id
        operType: 操作类型 3保存到通讯录 2从通讯录移除
        """
        request = saveContractListRequest(chatroomId=chatroomId, operType=operType)
        return type_validate_python(Response, resp_json(await self.call_api("/group/saveContractList", **model_dump(request))))
    
    async def adminOperate(self, chatroomId: str, operType: str, wxids: List[str]) -> Response:
        """
        群管理员操作
        chatroomId: 群聊id
        operType: 操作类型 1：添加群管理（可添加多个微信号） 2：删除群管理（可删除多个） 3：转让（只能转让一个微信号）
        """
        request = adminOperateRequest(chatroomId=chatroomId, operType=operType, wxids=wxids)
        return type_validate_python(Response, resp_json(await self.call_api("/group/adminOperate", **model_dump(request))))

    async def pinChat(self, chatroomId: str, top: bool) -> Response:
        """
        置顶聊天
        chatroomId: 群聊id
        top: 是否置顶
        """
        request = pinChatRequest(chatroomId=chatroomId, top=top)
        return type_validate_python(Response, resp_json(await self.call_api("/group/pinChat", **model_dump(request))))
    
    async def setMsgSilence(self, chatroomId: str, silence: bool) -> Response:
        """
        设置消息免打扰
        chatroomId: 群聊id
        silence: 是否免打扰
        """
        request = setMsgSilenceRequest(chatroomId=chatroomId, silence=silence)
        return type_validate_python(Response, resp_json(await self.call_api("/group/setMsgSilence", **model_dump(request))))

    async def joinRoomUsingQRCode(self, qrUrl: str) -> joinRoomUsingQRCodeResponse:
        """
        使用群聊二维码进群
        qrUrl: 通过解析群二维码图片获得的链接
        """
        request = joinRoomUsingQRCodeRequest(qrUrl=qrUrl)
        return type_validate_python(joinRoomUsingQRCodeResponse, resp_json(await self.call_api("/group/joinRoomUsingQRCode", **model_dump(request))))
    
    async def roomAccessApplyCheckApprove(self, chatroomId: str, newMsgId: str, msgContent: str) -> Response:
        """
        群聊进群申请审核
        chatroomId: 群聊id
        newMsgId: 消息id
        msgContent: 消息内容
        """
        request = roomAccessApplyCheckApproveRequest(chatroomId=chatroomId, newMsgId=newMsgId, msgContent=msgContent)
        return type_validate_python(Response, resp_json(await self.call_api("/group/roomAccessApplyCheckApprove", **model_dump(request))))
    
    async def revokeMsg(self, toWxid: str, msgId: str, newMsgId: str, createTime: str) -> Response:
        """
        撤回消息
        toWxidL: 好友/群的ID
        msgId: 回调中的msgId
        newMsgId: 回调中的newMsgId
        createTime: 回调中的createTime
        """
        request = revokeMsgRequest(toWxid=toWxid, msgId=msgId, newMsgId=newMsgId, createTime=createTime)
        return type_validate_python(Response, resp_json(await self.call_api("/message/revokeMsg", **model_dump(request))))

    async def addLabel(self, labelName: str) -> addLabelResponse:
        """
        添加标签
        labelName: 标签名称
        """
        request = addLabelRequest(labelName=labelName)
        return type_validate_python(addLabelResponse, resp_json(await self.call_api("/label/add", **model_dump(request))))
    
    async def delLabelRequest(self, labels: List[str]) -> Response:
        """
        删除标签
        labels: 标签id列表
        """
        labels = ",".join(labels)
        request = delLabelRequest(labels=labels)
        return type_validate_python(Response, resp_json(await self.call_api("/label/delete", **model_dump(request))))
    
    async def getLabelList(self) -> getLabelListResponse:
        """
        获取标签列表
        """
        return type_validate_python(getLabelListResponse, resp_json(await self.call_api("/label/list")))
    
    async def modifyMemberList(self, labelIds: List[str], wxIds: List[str]) -> Response:
        """
        修改标签
        labelIds: 标签id列表
        wxid: 好友id
        """
        labelIds = ",".join(labelIds)
        request = modifyMemberListRequest(labelIds=labelIds, wxIds=wxIds)
        return type_validate_python(Response, resp_json(await self.call_api("/label/modifyMemberList", **model_dump(request))))
    
    async def getProfile(self) -> getProfileResponse:
        """
        获取个人信息
        """
        return type_validate_python(getProfileResponse, resp_json(await self.call_api("/personal/getProfile")))
    
    async def getQrCode(self) -> getQrCodeResponse:
        """
        获取个人二维码
        """
        return type_validate_python(getQrCodeResponse, resp_json(await self.call_api("/personal/getQrCode")))

    async def getSafetyInfo(self) -> getSafetyInfoResponse:
        """
        获取设备记录
        """
        return type_validate_python(getSafetyInfoResponse, resp_json(await self.call_api("/personal/getSafetyInfo")))
    
    async def privacySettings(self, option: int, open: bool) -> Response:
        """
        隐私设置
        option: 4: 加我为朋友时需要验证 7: 向我推荐通讯录朋友 8: 添加我的方式 手机号 25: 添加我的方式 微信号 38: 添加我的方式 群聊 39: 添加我的方式 我的二维码 40: 添加我的方式 名片
        open: 是否开启
        """
        request = privacySettingsRequest(option=option, open=open)
        return type_validate_python(Response, resp_json(await self.call_api("/personal/privacySettings", **model_dump(request))))
    
    async def updateProfile(self, city: str, country: str, nickName: str, province: str, sex: int, signature: str) -> Response:
        """
        更新个人信息
        city: 城市
        country: 国家
        nickName: 昵称
        province: 省份
        sex: 性别 1:男 2:女
        signature: 个性签名
        """
        request = updateProfileRequest(city=city, country=country, nickName=nickName, province=province, sex=sex, signature=signature)
        return type_validate_python(Response, resp_json(await self.call_api("/personal/updateProfile", **model_dump(request))))

    async def updateHeadImg(self, headImgUrl: str) -> Response:
        """
        更新头像
        img: 图片地址
        """
        request = updateHeadImgRequest(headImgUrl=headImgUrl)
        return type_validate_python(Response, resp_json(await self.call_api("/personal/updateHeadImg", **model_dump(request))))
    
    async def syncFavorFolder(self, syncKey: str = "") -> syncFavorResponse:
        """
        同步收藏文件夹,响应结果中会包含已删除的的收藏夹记录, 通过flag=1来判断已删除
        syncKey: 翻页key, 首次传空, 获取下一页传接口返回的syncKey
        """
        request = syncFavorRequest(syncKey=syncKey)
        return type_validate_python(syncFavorResponse, resp_json(await self.call_api("/favor/sync", **model_dump(request))))

    async def getFavorContent(self, favId: str) -> getFavorContentResponse:
        """
        获取收藏内容
        favId: 收藏id
        """
        request = getFavorContentRequest(favId=favId)
        return type_validate_python(getFavorContentResponse, resp_json(await self.call_api("/favor/getContent", **model_dump(request))))

    async def deleteFavorFolder(self, favId: str) -> Response:
        """
        删除收藏夹
        favId: 收藏id
        """
        request = deleteFavorFolderRequest(favId=favId)
        return type_validate_python(Response, resp_json(await self.call_api("/favor/delete", **model_dump(request))))
